/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import javax.annotation.Nonnull;

/*******************************************************************************
 *
 * A small enhancement of a plain {@code List<MetadataItemHolder<Item>>} which
 * exposes a {@link #get()} method to quickly get a {@link List} of {@code Item}s.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataItemHolderList<Item> implements List<MetadataItemHolder<Item>>
  {
    @Nonnull
    private final List<MetadataItemHolder<Item>> delegate;

    /***************************************************************************
     *
     *
     *
     **************************************************************************/
    public MetadataItemHolderList (@Nonnull final List delegate)
      {
        this.delegate = delegate;
      }

    /***************************************************************************
     *
     * Returns the {@link List} of the contained {@code Item}s.
     *
     * @param  the list of items
     *
     **************************************************************************/
    public List<Item> get()
      {
        final List<Item> result = new ArrayList<Item>();

        for (final MetadataItemHolder<Item> holder : delegate)
          {
            result.add(holder.get());
          }

        return result;
      }

    /***************************************************************************
     *
     * DELEGATE METHODS FOLLOW
     *
     **************************************************************************/
    @Override
    public <T> T[] toArray(T[] arg0)
      {
        return delegate.toArray(arg0);
      }

    @Override
    public Object[] toArray()
      {
        return delegate.toArray();
      }

    @Override
    public List<MetadataItemHolder<Item>> subList(int arg0, int arg1)
      {
        return delegate.subList(arg0, arg1);
      }

    @Override
    public int size()
      {
        return delegate.size();
      }

    @Override
    public MetadataItemHolder<Item> set(int arg0, MetadataItemHolder<Item> arg1)
      {
        return delegate.set(arg0, arg1);
      }

    @Override
    public boolean retainAll(Collection<?> arg0)
      {
        return delegate.retainAll(arg0);
      }

    @Override
    public boolean removeAll(Collection<?> arg0)
      {
        return delegate.removeAll(arg0);
      }

    @Override
    public MetadataItemHolder<Item> remove(int arg0)
      {
        return delegate.remove(arg0);
      }

    @Override
    public boolean remove(Object arg0)
      {
        return delegate.remove(arg0);
      }

    @Override
    public ListIterator<MetadataItemHolder<Item>> listIterator(int arg0)
      {
        return delegate.listIterator(arg0);
      }

    @Override
    public ListIterator<MetadataItemHolder<Item>> listIterator()
      {
        return delegate.listIterator();
      }

    @Override
    public int lastIndexOf(Object arg0)
      {
        return delegate.lastIndexOf(arg0);
      }

    @Override
    public Iterator<MetadataItemHolder<Item>> iterator()
      {
        return delegate.iterator();
      }

    @Override
    public boolean isEmpty()
      {
        return delegate.isEmpty();
      }

    @Override
    public int indexOf(Object arg0)
      {
        return delegate.indexOf(arg0);
      }

    @Override
    public int hashCode()
      {
        return delegate.hashCode();
      }

    @Override
    public MetadataItemHolder<Item> get(int arg0)
      {
        return delegate.get(arg0);
      }

    @Override
    public boolean equals(Object arg0)
      {
        return delegate.equals(arg0);
      }

    @Override
    public boolean containsAll(Collection<?> arg0)
      {
        return delegate.containsAll(arg0);
      }

    @Override
    public boolean contains(Object arg0)
      {
        return delegate.contains(arg0);
      }

    @Override
    public void clear()
      {
        delegate.clear();
      }

    @Override
    public boolean addAll(int arg0, Collection<? extends MetadataItemHolder<Item>> arg1)
      {
        return delegate.addAll(arg0, arg1);
      }

    @Override
    public boolean addAll(Collection<? extends MetadataItemHolder<Item>> arg0)
      {
        return delegate.addAll(arg0);
      }

    @Override
    public void add(int arg0, MetadataItemHolder<Item> arg1)
      {
        delegate.add(arg0, arg1);
      }

    @Override
    public boolean add(MetadataItemHolder<Item> arg0)
      {
        return delegate.add(arg0);
      }

    @Override
    @Nonnull
    public String toString()
      {
        return delegate.toString();
      }
  }
